=== added file 'pics/stock_policy_button_dontstock.png'
Binary files pics/stock_policy_button_dontstock.png and pics/stock_policy_button_dontstock.png differ
=== added file 'pics/stock_policy_button_normal.png'
Binary files pics/stock_policy_button_normal.png and pics/stock_policy_button_normal.png differ
=== added file 'pics/stock_policy_button_prefer.png'
Binary files pics/stock_policy_button_prefer.png and pics/stock_policy_button_prefer.png differ
=== added file 'pics/stock_policy_button_remove.png'
Binary files pics/stock_policy_button_remove.png and pics/stock_policy_button_remove.png differ
=== added file 'pics/ware_list_bg_selected.png'
Binary files pics/ware_list_bg_selected.png and pics/ware_list_bg_selected.png differ
=== modified file 'src/wui/warehousewindow.cc'
--- src/wui/warehousewindow.cc	2010-10-29 22:01:38 +0000
+++ src/wui/warehousewindow.cc	2010-10-30 12:29:45 +0000
@@ -89,28 +89,84 @@
 			return false;
 
 		if (m_igbase.can_act(m_warehouse.owner().player_number())) {
-			Warehouse::StockPolicy policy = m_warehouse.get_stock_policy(get_type() == WORKER, ware);
-			Warehouse::StockPolicy newpolicy;
-
-			switch(policy) {
-			case Warehouse::SP_Normal: newpolicy = Warehouse::SP_Prefer; break;
-			case Warehouse::SP_Prefer: newpolicy = Warehouse::SP_DontStock; break;
-			case Warehouse::SP_DontStock: newpolicy = Warehouse::SP_Remove; break;
-			default: newpolicy = Warehouse::SP_Normal; break;
-			}
-
-			m_igbase.game().send_player_command
-				(*new Widelands::Cmd_SetStockPolicy
-					(m_igbase.game().get_gametime(), m_warehouse.owner().player_number(),
-					 m_warehouse, get_type() == WORKER, ware, newpolicy));
+			toggle_ware(ware);
 		}
-
 		return true;
 	}
 
 	return WaresDisplay::handle_mousepress(btn, x, y);
 }
 
+/**
+ * Wraps the wares display together with some buttons
+ */
+struct WarehouseWaresPanel : UI::Box {
+	WarehouseWaresPanel(UI::Panel * parent, uint32_t width, Interactive_GameBase &, Warehouse &, WaresDisplay::wdType type);
+
+	void set_policy(Warehouse::StockPolicy);
+private:
+	WarehouseWaresDisplay m_display;
+	Interactive_GameBase & m_gb;
+	Warehouse & m_wh;
+	WaresDisplay::wdType m_type;
+
+};
+
+WarehouseWaresPanel::WarehouseWaresPanel(UI::Panel * parent, uint32_t width, Interactive_GameBase & gb, Warehouse & wh, WaresDisplay::wdType type) :
+	UI::Box(parent, 0, 0, UI::Box::Vertical),
+	m_display(this, width, gb, wh, type),
+	m_gb(gb),
+	m_wh(wh),
+	m_type(type)
+{
+	add(&m_display, UI::Box::AlignLeft, true);
+
+	UI::Box *buttons = new UI::Box(this, 0, 0, UI::Box::Horizontal);
+	add(buttons, UI::Box::AlignLeft);
+		
+	if (m_gb.can_act(m_wh.owner().player_number())) {
+#define ADD_POLICY_BUTTON(policy, policyname)                             \
+        	buttons->add(new UI::Callback_Button(                     \
+			buttons, "workarea",                              \
+			0, 0, 34, 34,                                     \
+			g_gr->get_picture(PicMod_UI,"pics/but4.png"),     \
+			g_gr->get_picture(PicMod_Game,                    \
+		              "pics/stock_policy_button_" #policy ".png"),\
+			boost::bind(&WarehouseWaresPanel::set_policy,     \
+			            this, Warehouse::SP_##policyname),    \
+			_("Normal policy"))                               \
+		, UI::Box::AlignCenter);                                  \
+	
+		ADD_POLICY_BUTTON(normal, Normal)
+		ADD_POLICY_BUTTON(prefer, Prefer)
+		ADD_POLICY_BUTTON(dontstock, DontStock)
+		ADD_POLICY_BUTTON(remove, Remove)
+	}
+}
+
+/**
+ * Add Buttons policy buttons
+ */
+void WarehouseWaresPanel::set_policy(Warehouse::StockPolicy newpolicy) {
+	bool is_workers = m_type == WaresDisplay::WORKER;
+	Widelands::Ware_Index nritems =
+	                   is_workers ? m_wh.owner().tribe().get_nrworkers() :
+				        m_wh.owner().tribe().get_nrwares();
+	if (m_gb.can_act(m_wh.owner().player_number())) {
+	       for (Widelands::Ware_Index id = Widelands::Ware_Index::First();
+	            id < nritems; ++id) {
+			if (m_display.ware_selected(id)) {
+				m_gb.game().send_player_command
+					(*new Widelands::Cmd_SetStockPolicy
+						(m_gb.game().get_gametime(),
+						 m_wh.owner().player_number(),
+						 m_wh, is_workers,
+						 id, newpolicy));
+			}
+		}
+	}
+}
+
 
 /**
  * Status window for warehouses
@@ -149,8 +205,8 @@
 void Warehouse_Window::make_wares_tab
 	(WaresDisplay::wdType type, PictureID tabicon, const std::string & tooltip)
 {
-	WaresDisplay * display = new WarehouseWaresDisplay(get_tabs(), Width, igbase(), warehouse(), type);
-	get_tabs()->add("wares", tabicon, display, tooltip);
+	WarehouseWaresPanel * panel = new WarehouseWaresPanel(get_tabs(), Width, igbase(), warehouse(), type);
+	get_tabs()->add("wares", tabicon, panel, tooltip);
 }
 
 /**

=== modified file 'src/wui/waresdisplay.cc'
--- src/wui/waresdisplay.cc	2010-10-30 09:18:16 +0000
+++ src/wui/waresdisplay.cc	2010-10-30 12:29:45 +0000
@@ -42,7 +42,10 @@
 		(this,
 		 0, get_inner_h() - 25, get_inner_w(), 20,
 		 _("Stock"), UI::Align_Center),
-	m_type (WORKER)
+	//TODO need to know m_type at construction time
+	m_type (WORKER),
+	m_selected(m_type == WORKER ? m_tribe.get_nrworkers()
+	                            : m_tribe.get_nrwares(), false)
 
 {
 }
@@ -206,7 +209,10 @@
 
 	//  draw a background
 	const PictureID picid =
-		g_gr->get_picture(PicMod_Game, "pics/ware_list_bg.png");
+		g_gr->get_picture(PicMod_Game,
+			ware_selected(id) ?  "pics/ware_list_bg_selected.png"
+			                  :  "pics/ware_list_bg.png"
+		);
 	uint32_t w, h;
 	g_gr->get_picture_size(picid, w, h);
 
@@ -235,3 +241,14 @@
 		 buffer,
 		 UI::Align_Right);
 }
+
+// Wares highlighting/selecting
+void WaresDisplay::select_ware(Widelands::Ware_Index ware) {
+	m_selected[ware] = true;
+}
+void WaresDisplay::unselect_ware(Widelands::Ware_Index ware) {
+	m_selected[ware] = false;
+}
+bool WaresDisplay::ware_selected(Widelands::Ware_Index ware) {
+	return	m_selected[ware];
+}

=== modified file 'src/wui/waresdisplay.h'
--- src/wui/waresdisplay.h	2010-10-30 09:14:05 +0000
+++ src/wui/waresdisplay.h	2010-10-30 12:29:45 +0000
@@ -56,6 +56,22 @@
 	void add_warelist(Widelands::WareList const &, wdType);
 	void remove_all_warelists();
 
+	// Wares may be selected (highlighted)
+	void select_ware(Widelands::Ware_Index);
+	void unselect_ware(Widelands::Ware_Index);
+	bool ware_selected(Widelands::Ware_Index);
+	void toggle_ware(Widelands::Ware_Index ware) {
+		if (ware_selected(ware))
+			unselect_ware(ware);
+		else
+			select_ware(ware);
+	}
+
+	// Wares may be hidden
+	void hide_ware(Widelands::Ware_Index);
+	void unhide_ware(Widelands::Ware_Index);
+	bool ware_hidden(Widelands::Ware_Index);
+
 	Widelands::Ware_Index ware_at_point(int32_t x, int32_t y) const;
 	wdType get_type() const {return m_type;}
 
@@ -74,11 +90,14 @@
 
 private:
 	typedef std::vector<Widelands::WareList const *> vector_type;
+	typedef std::vector<bool> selection_type;
 
 	Widelands::Tribe_Descr const & m_tribe;
 	UI::Textarea        m_curware;
 	wdType              m_type;
 	vector_type         m_warelists;
+	selection_type      m_selected;
+	selection_type      m_hidden;
 };
 
 #endif

